import 'dart:collection';
import 'dart:io';

import 'package:app_tracking_transparency/app_tracking_transparency.dart';
import 'package:cramr_ui_lib/ads/preloaded_banner_ad.dart';
import 'package:flutter/foundation.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

class AdsController {
  final MobileAds _instance;

  bool _isPersonalisedAllowed = false;

  bool get isPersonalisedAllowed => _isPersonalisedAllowed;

  Map<String, PreloadedBannerAd> _preloadedAd = HashMap();

  AdRequest androidAdRequest = const AdRequest();
  AdRequest iosPersonalisedAdRequest =
      const AdRequest(nonPersonalizedAds: false);
  AdRequest iosNonPersonalisedAdRequest =
      const AdRequest(nonPersonalizedAds: true);

  AdsController(this._instance);

  AdRequest createAdRequest() {
    AdRequest adRequest = androidAdRequest;
    if (Platform.isIOS) {
      if (isPersonalisedAllowed) {
        if (kDebugMode) {
          print('personalised IOS ad request');
        }
        adRequest = iosPersonalisedAdRequest;
      } else {
        if (kDebugMode) {
          print('NON personalised IOS ad request');
        }
        adRequest = iosNonPersonalisedAdRequest;
      }
    }
    return adRequest;
  }

  void dispose(String adUnitId) {
    _preloadedAd[adUnitId]?.dispose();
    _preloadedAd.remove(adUnitId);
  }

  Future<void> initialize() async {
    _instance.initialize();
    Future<TrackingStatus> status = isTrackingAllowed();
    status.then((value) {
      _isPersonalisedAllowed = (TrackingStatus.authorized == value);
    });
  }

  Future<TrackingStatus> isTrackingAllowed() async {
    final status = await AppTrackingTransparency.requestTrackingAuthorization();
    return status;
  }

  /// Starts preloading an ad to be used later.
  ///
  /// The work doesn't start immediately so that calling this doesn't have
  /// adverse effects (jank) during start of a new screen.
  void preloadAd(String adUnitId) {
    if (kDebugMode) {
      print('preloading ad with id $adUnitId');
    }

    // Wait a bit so that calling at start of a new screen doesn't have
    // adverse effects on performance.
    Future<void>.delayed(const Duration(milliseconds: 500)).then((_) {
      _preloadedAd[adUnitId] =
          PreloadedBannerAd(size: AdSize.mediumRectangle, adUnitId: adUnitId);
      return _preloadedAd[adUnitId]?.load();
    });
  }

  /// Allows caller to take ownership of a [PreloadedBannerAd].
  ///
  /// If this method returns a non-null value, then the caller is responsible
  /// for disposing of the loaded ad.
  PreloadedBannerAd? takePreloadedAd(String adUnitId) {
    final ad = _preloadedAd[adUnitId];
    _preloadedAd[adUnitId]?.dispose();
    _preloadedAd.remove(adUnitId);
    return ad;
  }
}
