import 'dart:async';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:google_mobile_ads/google_mobile_ads.dart';

class PreloadedBannerAd {

  /// Something like [AdSize.mediumRectangle].
  final AdSize size;

  late final AdRequest _adRequest;

  BannerAd? _bannerAd;

  final String adUnitId;

  final _adCompleter = Completer<BannerAd>();

  PreloadedBannerAd({
    required this.size,
    required this.adUnitId,
    AdRequest? adRequest,
  }) {
    if (kDebugMode) {
      print('***************************** PreloadBannerAd constructor');
      print('***************************** adUnitId= $adUnitId');
    }
    _adRequest = adRequest ?? const AdRequest();
  }

  Future<BannerAd> get ready => _adCompleter.future;

  Future<void> load() {
    assert(Platform.isAndroid || Platform.isIOS,
        'AdMob currently does not support ${Platform.operatingSystem}');

    if (kDebugMode) {
      print(adUnitId);
      print('loading preloaded banner ad');
    }

    _bannerAd = BannerAd(
      // This is a test ad unit ID from
      // https://developers.google.com/admob/android/test-ads. When ready,
      // you replace this with your own, production ad unit ID,
      // created in https://apps.admob.com/.
      adUnitId: adUnitId,
      size: size,
      request: _adRequest,
      listener: BannerAdListener(
        onAdLoaded: (Ad ad) {
          _adCompleter.complete(_bannerAd);
        },
        onAdFailedToLoad: (Ad ad, LoadAdError error) {
          if (kDebugMode) {
            print('Banner failedToLoad: $error');
          }
          // TODO: figure out whether this is actually needed
          //  _adCompleter.completeError(error);
          ad.dispose();
        },
        onAdImpression: (Ad ad) {
          if (kDebugMode) {
            print('Ad impression registered');
          }
        },
        onAdClicked: (Ad ad) {
        },
      ),
    );

    return _bannerAd!.load();
  }

  void dispose() {
    _bannerAd?.dispose();
  }
}
